==> cosmo_1.0.tar.gz: cosmo/src/precalc.c <==
/*****************************************************************************
*  			     					             *
*      cosmo - Constrained search for motifs in DNA sequences                *
*      Copyright (C) 2006  Oliver Bembom (bembom@berkeley.edu)               *
*                                                                            *
*      This file is part of cosmo.                                           *
*                                                                            *
*      cosmo is free software; you can redistribute it and/or                *
*      modify it under the terms of the GNU Lesser General Public            *
*      License as published by the Free Software Foundation; either          *
*      version 2.1 of the License, or (at your option) any later version.    *
*                                                                            *
*      cosmo is distributed in the hope that it will be useful,              *
*      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
*      Lesser General Public License for more details.                       *
*                                                                            *
*      You should have received a copy of the GNU Lesser General Public      *
*      License along with this program; if not, write to the Free Software   *
*      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,            *
*      MA  02110-1301, USA.                                                  *
*                                                                            *
*                                                                            *
*****************************************************************************/


/*****************************************************************************
 *                                                                           *
 *      This module contains routines for precalculating various quantities  *
 *      before the opimization is run                                        *
 *                                                                           *
 ****************************************************************************/

#include <R_ext/PrtUtil.h>
#include <R_ext/Memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "macros.h"
#include "cosmo.h"
#include "matLib.h"
#include "precalc.h"
  


/**********************************************************************/
/*
  precalcBack

  Precalculate the contribution of the background model to the likeihood
  function for all possible motif sites.
	
*/
/**********************************************************************/
int precalcBack(
		 DATASET *dataSet   /* the dataset */
		 )
{

  getLogCumBack(dataSet);
  getBackLR(dataSet);
  getCumProbBack(dataSet);

  return(1);
}

/**********************************************************************/
/*
  findIndex

  Find the row for a given oligo in that subpart of the transition
  matrix that contains the transition probabilities for the
  requested order.

  Within each such part, oligos are arranged as

  AA
  AC
  AG
  AT
  CA
  CG 
  ...
  TA
  TC
  TG
  TT

  for example 

  Returns the row number relative to the appropriate subpart.
	
*/
/**********************************************************************/
long findIndex(
	       int *kTuple,          /* the oligo for which we need the corresponding row */
	       int order             /* the length of the oligo */
	       )
{
  long i, index;

  index=0;
  if (order>0){
    for (i=0;i<order;i++){
      index += pow(4,order-i-1)*kTuple[i]; 
    }
  }

  return index;
}


/**********************************************************************/
/*
  getCondBackProb

  Calculate a vector of letter probabilities given the previous k nucleotides
  that is stored in backProb on exit.
	
*/
/**********************************************************************/
int getCondBackProb(
		     int seqNum,        /* the sequence in which the letter is */
		     int pos,           /* the position in this sequence */
		     double *backProb,  /* the background probabilities */
		     DATASET *dataSet   /* the dataset */
		     )
{
  
  int index, *seq, nuc, order;
  double *transMat;


  seq             = dataSet->samples[seqNum].seq;
  order           = dataSet->order;
  
  /* the row with the right probabilities */
  if(pos<order){	    
    transMat = dataSet->transMats[pos];
    index    = findIndex(seq,pos);
    for (nuc=0; nuc<pos; nuc++) Rprintf("%d",*(seq+nuc));
    Rprintf("\n");
  }else{
    transMat = dataSet->transMats[order];
    index    = findIndex(seq+pos-order, order);
    for (nuc=0; nuc<order; nuc++) Rprintf("%d",*(seq+pos-order+nuc));
    Rprintf("\n");
  }

  for (nuc=0;nuc<4;nuc++) backProb[nuc] = exp(transMat[4*index+nuc]);


  if (DEBUG>=4){
    Rprintf("getCondBack:\n");    
    PrintDoubleMatrix2File(stderr,backProb,1,4);
  }

  return(1);
}

/**********************************************************************/
/*
  getLogProbBack

  Calculate the (log) probability of nucleotide in position 'pos' of 
  sequence 'seqNum' under background model

	
*/
/**********************************************************************/
double getLogProbBack(
		      DATASET *dataSet,   /* the dataset */
		      int seqNum,         /* the sequence number */
		      int pos             /* position in the sequence */
		      )
{


  int *seq;                 /* sequence */
  double *transMat;         /* transition matrix */
  int order;                /* order of background Markov model */
  int index;

  /* Error checking */
  if (seqNum > dataSet->numSeqs-1){
    Rprintf("getLogProbBack: Dataset has only %d sequences. Requested sequence %d does not exit. Exiting ...\n", 
	   dataSet->numSeqs, seqNum+1);
    return(0);
  }

  if (pos > dataSet->samples[seqNum].length){
    Rprintf("getLogProbBack: Sequence %d only has %ld nucleotides. Position %d does not exist. Exiting ...\n", 
	   seqNum, dataSet->samples[seqNum].length, pos+1);
    return(0);
  }
    
  seq             = dataSet->samples[seqNum].seq;
  order           = dataSet->order;

  if (seq[pos]<4){
    /* regular character */
    if(pos<order){	    
      transMat = dataSet->transMats[pos];
      index    = findIndex(seq,pos+1);
    }else{
      transMat = dataSet->transMats[order];
      index    = findIndex(seq+pos-order, order+1);
    }
    return transMat[index];
  }else{
    /* unknown character 'X' */
    return LOG(1.0/4);
  }
  

}
 
/**********************************************************************/
/*
  getLogCumBack

  Calculate cumumulative log Probabilities under background model.
  Specificically, calculate the probabilities of all nucleotides
  up to but not including the current one under the background model.
  Then the log probabiliity of any stretch of sequence under the
  background model can be obtained as the difference of the
  two cumulative log probabilities.
	
*/
/**********************************************************************/
int getLogCumBack(
		   DATASET *dataSet    /* the dataset */
		   )
{

  int seqNum, pos;          /* various indices */
  int *seq;                 /* sequence */
  long seqLength; 
  double *logCumBack;       
  int order;                /* order of background Markov model */


  order           = dataSet->order;

  for (seqNum=0; seqNum<dataSet->numSeqs;seqNum++){

    logCumBack    = dataSet->samples[seqNum].logCumBack;
    seq           = dataSet->samples[seqNum].seq;
    seqLength     = dataSet->samples[seqNum].length;

    if (DEBUG>=5) Rprintf("getLogCumBack: seqNum = %d seqLength = %ld\n", seqNum, seqLength);

    logCumBack[0] = 0;
    for(pos=0;pos<seqLength;pos++){
      logCumBack[pos+1] = logCumBack[pos] + getLogProbBack(dataSet,seqNum,pos);   
    }

  }	
    
  if (DEBUG>=3){
    Rprintf("getLogCumBack:\n"); 
    for (seqNum=0; seqNum<dataSet->numSeqs;seqNum++){
      Rprintf("Seq %d:\n", seqNum);
      PrintDoubleMatrix2File(stderr,dataSet->samples[seqNum].logCumBack,1,dataSet->samples[seqNum].length+1);
    }
  }

  return(1);
}



/**********************************************************************/
/*
  getBackLR

  Calculate contribution to likelihood ratio of nucleotides 
  belonging to the background assuming that a motif starts in 
  a given position. 
	

  Calculate backLR for sequence n at position l as 

  1/(Probability under background model for stretch from postion l to l + w)

*/
/**********************************************************************/
int getBackLR(
	       DATASET *dataSet  /* the datset */
	       )
{

  int width;               /* current motif width */
  int seqNum;              /* current sequence number */
  int pos;                 /* position */
  int end;                 /* end */ 

  double *backLR;          /* on exit: probability contributions of background given that motif starts at certain position */
  double *logCumBack;      /* cumulative log probabilities under background model */
  int minWidth;            /* lower bound on motif widths to consider */
  int maxWidth;            /* upper bound on motif widths to consider */
  long seqLength;          /* sequence lengths */
  int order;               /* order of background Markov model */
  long numSeqs;            /* number of sequences */


  minWidth = dataSet->minWidth;
  maxWidth = dataSet->maxWidth;
  order    = dataSet->order;
  numSeqs  = dataSet->numSeqs;


  for (seqNum = 0; seqNum < numSeqs; seqNum++){
    backLR      = dataSet->samples[seqNum].backLR;
    logCumBack  = dataSet->samples[seqNum].logCumBack;
    seqLength   = dataSet->samples[seqNum].length;
      
    for (width = minWidth; width<=maxWidth;width++){

      for (pos = 0; pos < seqLength - width + 1;pos++){
	end = MIN(pos+width+order-1,seqLength-1);
	backLR[(width-minWidth)*seqLength + pos] = exp(logCumBack[pos] - logCumBack[pos+width]);
      }

    }
  }

  if (DEBUG>=3){
    Rprintf("getBackLR:\n");
    for (seqNum = 0; seqNum < numSeqs; seqNum++){
      backLR      = dataSet->samples[seqNum].backLR;
      seqLength   = dataSet->samples[seqNum].length;
     
      Rprintf("Seq %d:\n", seqNum);
      for (width = minWidth; width<=maxWidth;width++){
	Rprintf("width = %d\n", width);
	PrintDoubleMatrixE(backLR + (width-minWidth)*seqLength, 1, seqLength-width+1);
      }
    }
  }

  return(1);
}




/**********************************************************************/
/*
  getCumProbBack

  Calculate the probability for all width W oligos under the backgound
  model, for all widths W.
	
*/
/**********************************************************************/
int getCumProbBack(
		    DATASET *dataSet  /* the dataset */
		    )
{

  int seqNum, start, width;   /* various indices */
  int minWidth;
  int maxWidth;
  int numSeqs;
  int seqLength;
  double *cumProbBack;
  double *logCumBack;

  minWidth   = dataSet->minWidth;
  maxWidth   = dataSet->maxWidth;
  numSeqs    = dataSet->numSeqs;

  for (seqNum=0; seqNum<numSeqs;seqNum++){
    cumProbBack = dataSet->samples[seqNum].cumProbBack;
    seqLength   = dataSet->samples[seqNum].length;
    logCumBack  = dataSet->samples[seqNum].logCumBack;

    for (width = minWidth; width<=maxWidth; width++){
      for(start=0;start<seqLength - width + 1;start++){
	cumProbBack[(width-minWidth)*seqLength + start] = exp(logCumBack[start+width]-logCumBack[start]);
      }
    }
  }    


  if (DEBUG>=3){
    Rprintf("getCumProbBack:\n");
    for (seqNum = 0; seqNum < numSeqs; seqNum++){
      cumProbBack = dataSet->samples[seqNum].cumProbBack;
      seqLength   = dataSet->samples[seqNum].length;
     
      Rprintf("Seq %d:\n", seqNum);
      for (width = minWidth; width<=maxWidth;width++){
	Rprintf("width = %d\n", width);
	PrintDoubleMatrixE(cumProbBack + (width-minWidth)*seqLength, 1, seqLength-width+1);
      }
    }
  }

  return(1);
}

==> cosmo_1.6.0.tar.gz: cosmo/src/precalc.c <==
/******************************************************************************
*                                                                             *
*      cosmo - Constrained search for motifs in DNA sequences                 *
*      Author: Oliver Bembom                                                  *
*                                                                             *
*      This file is part of cosmo.                                            *
*                                                                             *
*      Conditions of use:                                                     *
*                                                                             *
*    1. cosmo is under the exclusive copyright of Oliver Bembom,              *
*       (bembom@berkeley.edu), the Regents of the University of California,   *
*       and Peter Spellucci (spellucci@mathematik.tu-darmstadt.de).           *
*                                                                             *
*    2. Permission to use, copy, modify, and distribute any part of this      *
*       software for research purposes, without fee and without a written     *
*       agreement, is hereby granted, provided that this license appears in   *
*       all copies. Commercial uses require permission and licensing from     *
*       Oliver Bembom, the Regents of the University of California, and       *
*       Peter Spellucci.                                                      *
*                                                                             *
*    3. If you use this software in your research, please cite                *
*                                                                             *
*       O. Bembom, S. Keles, M.J. van der Laan (2007). Supervised Detection   *
*       of Conserved Motifs in DNA Sequences with cosmo. Statistical          *
*       Applications in Genetics and Molecular Biology: Vol. 6 : Iss. 1,      *
*       Article 8. http://www.bepress.com/sagmb/vol6/iss1/art8.               *
*                                                                             *
*       cosmo makes use of the donlp2() function by Peter Spellucci.          *
*       The use of donlp2() must be acknowledged in any publication which     *
*       contains results obtained with cosmo or parts of it. Citation of the  *
*       author's name and netlib-source is suitable.                          *
*                                                                             *
*    4. In no event shall the author be liable to any party for direct, in-   *
*       direct, special, incidental, or consequential damages, including lost *
*       profits, arising out of the use of this software, even if the author  *
*       has been advised of the possibility of such damage.                   *
*                                                                             *
*    5. cosmo is provided on an 'as is' basis, and the author has no obli-    *
*       gations to provide maintenance, support, updates, enhancements, or    *
*       modifications. The author makes no representations and extends no     *
*       warranties of any kind, either expressed or implied, including, but   *
*       not limited to, the implied warranties of merchantability or fitness  *
*       for a particular purpose, or that the use of the material will not    *
*       infringe any patent, trademark or other rights.                       *
*                                                                             *
*                                                                             *
******************************************************************************/


/*****************************************************************************
 *                                                                           *
 *      This module contains routines for precalculating various quantities  *
 *      before the opimization is run                                        *
 *                                                                           *
 ****************************************************************************/

#include <R_ext/PrtUtil.h>
#include <R_ext/Memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "macros.h"
#include "cosmo.h"
#include "matLib.h"
#include "precalc.h"
#include "back.h"  


/**********************************************************************/
/*
  precalcBack

  Precalculate the contribution of the background model to the likeihood
  function for all possible motif sites.
	
*/
/**********************************************************************/
int precalcBack(
		 DATASET *dataSet   /* the dataset */
		 )
{

  getLogCumBack(dataSet);
  getBackLR(dataSet);
  getCumProbBack(dataSet);

  return(1);
}

/**********************************************************************/
/*
  findIndex

  Find the row for a given oligo in that subpart of the transition
  matrix that contains the transition probabilities for the
  requested order.

  Within each such part, oligos are arranged as

  AA
  AC
  AG
  AT
  CA
  CG 
  ...
  TA
  TC
  TG
  TT

  for example 

  Returns the row number relative to the appropriate subpart.
	
*/
/**********************************************************************/
long findIndex(
	       int *kTuple,          /* the oligo for which we need the corresponding row */
	       int order             /* the length of the oligo */
	       )
{
  long i, index;

  index=0;
  if (order>0){
    for (i=0;i<order;i++){
      index += pow(4,order-i-1)*kTuple[i]; 
    }
  }

  return index;
}


/**********************************************************************/
/*
  getLogProbBack

  Calculate the (log) probability of nucleotide in position 'pos' of 
  sequence 'seqNum' under background model

	
*/
/**********************************************************************/
double getLogProbBack(
		      DATASET *dataSet,   /* the dataset */
		      int seqNum,         /* the sequence number */
		      int pos,            /* position in the sequence */
		      int order,          /* order to use for background model */
		      BOOLEAN back        /* background sequence? */
		      )
{


  int *seq;                 /* sequence */
  double *transMat;         /* transition matrix */
  int index;

    
  seq = back ? dataSet->backSamples[seqNum].seq : dataSet->samples[seqNum].seq;

  if (seq[pos]<4){
    /* regular character */
    if(pos<order){	    
      if (containsX(seq, pos+1, 4)){
	transMat = dataSet->transMats[0];
	index    = findIndex(seq+pos,1);
      }else{
	transMat = dataSet->transMats[pos];
	index    = findIndex(seq,pos+1);
      }
    }else{
      if (containsX(seq+pos-order, order+1, 4)){
	transMat = dataSet->transMats[0];
	index    = findIndex(seq+pos,1);
      }else{
	transMat = dataSet->transMats[order];
	index    = findIndex(seq+pos-order, order+1);
      }
    }
    if (COSMO_MSG_LEVEL>=4){
      Rprintf( "seqNum = %d, pos = %d, index = %d, logProb = %e\n", seqNum, pos, index, transMat[index]);
    }

    return transMat[index];
  }else{
    /* unknown character 'X' */
    if (COSMO_MSG_LEVEL>=4) Rprintf( "seqNum = %d, pos = %d, logProb = %e\n", seqNum, pos, log(0.25));

    return log(0.25);
  }
  

}
 
/**********************************************************************/
/*
  getLogCumBack

  Calculate cumumulative log Probabilities under background model.
  Specificically, calculate the probabilities of all nucleotides
  up to but not including the current one under the background model.
  Then the log probabiliity of any stretch of sequence under the
  background model can be obtained as the difference of the
  two cumulative log probabilities.
	
*/
/**********************************************************************/
int getLogCumBack(
		   DATASET *dataSet    /* the dataset */
		   )
{

  int seqNum, pos;          /* various indices */
  int *seq;                 /* sequence */
  long seqLength; 
  double *logCumBack;       
  int order;                /* order of background Markov model */


  order           = dataSet->order;

  for (seqNum=0; seqNum<dataSet->numSeqs;seqNum++){

    logCumBack    = dataSet->samples[seqNum].logCumBack;
    seq           = dataSet->samples[seqNum].seq;
    seqLength     = dataSet->samples[seqNum].length;

    if (COSMO_MSG_LEVEL>=5) Rprintf("getLogCumBack: seqNum = %d seqLength = %ld\n", seqNum, seqLength);

    logCumBack[0] = 0;
    for(pos=0;pos<seqLength;pos++){
      logCumBack[pos+1] = logCumBack[pos] + getLogProbBack(dataSet,seqNum,pos,order,FALSE);   
    }

  }	
    
  if (COSMO_MSG_LEVEL>=3){
    Rprintf("getLogCumBack:\n"); 
    for (seqNum=0; seqNum<dataSet->numSeqs;seqNum++){
      Rprintf("Seq %d:\n", seqNum);
      PrintDoubleMatrix2File(stderr,dataSet->samples[seqNum].logCumBack,1,dataSet->samples[seqNum].length+1);
    }
  }

  return(1);
}



/**********************************************************************/
/*
  getBackLR

  Calculate contribution to likelihood ratio of nucleotides 
  belonging to the background assuming that a motif starts in 
  a given position. 
	

  Calculate backLR for sequence n at position l as 

  1/(Probability under background model for stretch from postion l to l + w)

*/
/**********************************************************************/
int getBackLR(
	       DATASET *dataSet  /* the datset */
	       )
{

  int width;               /* current motif width */
  int seqNum;              /* current sequence number */
  int pos;                 /* position */
  int end;                 /* end */ 

  double *backLR;          /* on exit: probability contributions of background given that motif starts at certain position */
  double *logCumBack;      /* cumulative log probabilities under background model */
  int minWidth;            /* lower bound on motif widths to consider */
  int maxWidth;            /* upper bound on motif widths to consider */
  long seqLength;          /* sequence lengths */
  int order;               /* order of background Markov model */
  long numSeqs;            /* number of sequences */


  minWidth = dataSet->minWidth;
  maxWidth = dataSet->maxWidth;
  order    = dataSet->order;
  numSeqs  = dataSet->numSeqs;


  for (seqNum = 0; seqNum < numSeqs; seqNum++){
    backLR      = dataSet->samples[seqNum].backLR;
    logCumBack  = dataSet->samples[seqNum].logCumBack;
    seqLength   = dataSet->samples[seqNum].length;
      
    for (width = minWidth; width<=maxWidth;width++){

      for (pos = 0; pos < seqLength - width + 1;pos++){
	end = MIN(pos+width+order-1,seqLength-1);
	backLR[(width-minWidth)*seqLength + pos] = exp(logCumBack[pos] - logCumBack[pos+width]);
      }

    }
  }

  if (COSMO_MSG_LEVEL>=3){
    Rprintf("getBackLR:\n");
    for (seqNum = 0; seqNum < numSeqs; seqNum++){
      backLR      = dataSet->samples[seqNum].backLR;
      seqLength   = dataSet->samples[seqNum].length;
     
      Rprintf("Seq %d:\n", seqNum);
      for (width = minWidth; width<=maxWidth;width++){
	Rprintf("width = %d\n", width);
	PrintDoubleMatrixE(backLR + (width-minWidth)*seqLength, 1, seqLength-width+1);
      }
    }
  }

  return(1);
}




/**********************************************************************/
/*
  getCumProbBack

  Calculate the probability for all width W oligos under the backgound
  model, for all widths W.
	
*/
/**********************************************************************/
int getCumProbBack(
		    DATASET *dataSet  /* the dataset */
		    )
{

  int seqNum, start, width;   /* various indices */
  int minWidth;
  int maxWidth;
  int numSeqs;
  int seqLength;
  double *cumProbBack;
  double *logCumBack;

  minWidth   = dataSet->minWidth;
  maxWidth   = dataSet->maxWidth;
  numSeqs    = dataSet->numSeqs;

  for (seqNum=0; seqNum<numSeqs;seqNum++){
    cumProbBack = dataSet->samples[seqNum].cumProbBack;
    seqLength   = dataSet->samples[seqNum].length;
    logCumBack  = dataSet->samples[seqNum].logCumBack;

    for (width = minWidth; width<=maxWidth; width++){
      for(start=0;start<seqLength - width + 1;start++){
	cumProbBack[(width-minWidth)*seqLength + start] = exp(logCumBack[start+width]-logCumBack[start]);
      }
    }
  }    


  if (COSMO_MSG_LEVEL>=3){
    Rprintf("getCumProbBack:\n");
    for (seqNum = 0; seqNum < numSeqs; seqNum++){
      cumProbBack = dataSet->samples[seqNum].cumProbBack;
      seqLength   = dataSet->samples[seqNum].length;
     
      Rprintf("Seq %d:\n", seqNum);
      for (width = minWidth; width<=maxWidth;width++){
	Rprintf("width = %d\n", width);
	PrintDoubleMatrixE(cumProbBack + (width-minWidth)*seqLength, 1, seqLength-width+1);
      }
    }
  }

  return(1);
}
